import functools
import os
import sys

import botocore
import botocore.errorfactory
import click

from nimbo import CONFIG
from nimbo.core.config import RequiredCase
from nimbo.core.constants import (
    IS_TEST_ENV,
    NIMBO_DEFAULT_CONFIG,
)
from nimbo.core.print import nprint


def assert_required_config(*cases: RequiredCase):
    """
    Decorator for ensuring that required config is present
    """

    def decorator(func):
        @functools.wraps(func)
        def decorated(*args, **kwargs):
            try:
                CONFIG.assert_required_config_exists(*cases)
                return func(*args, **kwargs)
            except AssertionError as e:
                nprint(e, style="error")
                sys.exit(1)
            except FileNotFoundError as e:
                # Happens when nimbo config file is not found
                nprint(e, style="error")
                sys.exit(1)

        return decorated

    return decorator


def handle_errors(func):
    """
    Decorator for catching boto3 ClientErrors, ValueError or KeyboardInterrupts.
    In case of error print the error message and stop Nimbo.
    """

    @functools.wraps(func)
    def decorated(*args, **kwargs):
        if IS_TEST_ENV:
            return func(*args, **kwargs)
        else:
            try:
                return func(*args, **kwargs)
            except botocore.errorfactory.ClientError as e:
                nprint(e, style="error")
                sys.exit(1)
            except ValueError as e:
                nprint(e, style="error")
                sys.exit(1)
            except KeyboardInterrupt:
                print("Aborting...")
                sys.exit(1)

    return decorated


def generate_config(quiet=False) -> None:
    """ Create an example Nimbo config in the project root """

    if os.path.isfile(CONFIG.nimbo_config_file):
        should_overwrite = click.confirm(
            f"{CONFIG.nimbo_config_file} already exists, do you want to overwrite it?"
        )
        if not should_overwrite:
            print("Leaving Nimbo config intact")
            return

    with open(CONFIG.nimbo_config_file, "w") as f:
        f.write(NIMBO_DEFAULT_CONFIG)

    if not quiet:
        print(f"Example config written to {CONFIG.nimbo_config_file}")



import re
import sys

import pkg_resources
import pydantic

import nimbo.core.config
import nimbo.tests.aws.config
from nimbo.core.constants import IS_TEST_ENV

version = pkg_resources.get_distribution("nimbo").version


try:
    if IS_TEST_ENV:
        CONFIG = nimbo.tests.aws.config.make_config()
    else:
        CONFIG = nimbo.core.config.make_config()
except pydantic.error_wrappers.ValidationError as e:
    e_msg = str(e)
    e_num = len(e.errors())
    title_end = e_msg.index("\n", 1)
    new_title = f"{e_num} error{'' if e_num == 1 else 's'} in nimbo-config.yml\n"
    print(new_title + re.sub(r"\(type=.*\)", "", e_msg[title_end:]))
    sys.exit(1)



from nimbo import CONFIG
from nimbo.core.cloud_provider.provider_impl.aws.aws_provider import AwsProvider
from nimbo.core.cloud_provider.provider_impl.gcp.gcp_provider import GcpProvider
from nimbo.core.config.common_config import CloudProvider

if CONFIG.cloud_provider == CloudProvider.AWS:
    Cloud = AwsProvider()
else:
    Cloud = GcpProvider()
